#include <windows.h>
#include <winioctl.h>
#include <winsock.h>
#include <stdio.h>
#include <conio.h>

#include "..\fwctl.h"

CHAR	fwDevice[] = "\\\\.\\\\Fwleaf";

PCHAR strP(PCHAR msgbuf, USHORT port)
{
	port=(port>>8)|((port&0xFF)<<8);	// swap bytes
	if (port) sprintf(msgbuf,"%u",port);
	else lstrcpy(msgbuf,"any_port");
	return msgbuf;
}


PCHAR strIPM(PCHAR msgbuf, ULONG ip, ULONG mask)
{
	if (mask)
	{
		sprintf(msgbuf,"%u.%u.%u.%u/%u.%u.%u.%u",
			ip&0xFF,(ip>>8)&0xFF,(ip>>16)&0xFF,(ip>>24)&0xFF,
			mask&0xFF,(mask>>8)&0xFF,(mask>>16)&0xFF,(mask>>24)&0xFF);
	}
	else lstrcpy(msgbuf,"any_ip");
	return msgbuf;
}

VOID prnRules(PFWRECS pfr)
{
	ULONG	cnt;
	CHAR	sIP[64],sP[64],dIP[64],dP[64];

	for(cnt=0;cnt<pfr->num;cnt++)
	{
		printf("R[%.2u]: %s(%s) -> %s(%s) | %s\n",
			cnt,
			strIPM(sIP,pfr->fwr[cnt].sIP,pfr->fwr[cnt].sM),
			strP(sP,pfr->fwr[cnt].sP),
			strIPM(dIP,pfr->fwr[cnt].dIP,pfr->fwr[cnt].dM),
			strP(dP,pfr->fwr[cnt].dP),
			(pfr->fwr[cnt].ad)?"deny":"allow");
	}
}

VOID __cdecl main(INT argc, CHAR *argv[])
{
	HANDLE		hFw=INVALID_HANDLE_VALUE;
    ULONG       tmp,BytesReturned;
	FWRECS      setfwr={ 0 }, getfwr={ 0 };

	printf("FwLeaf test sample, 2007\n");

    hFw=CreateFile(	fwDevice,
					GENERIC_READ|GENERIC_WRITE,
					0,
					NULL,
					OPEN_EXISTING,
					FILE_ATTRIBUTE_NORMAL,
					(HANDLE)INVALID_HANDLE_VALUE);
    if ((hFw=CreateFile(	fwDevice,
					GENERIC_READ|GENERIC_WRITE,
					0,
					NULL,
					OPEN_EXISTING,
					FILE_ATTRIBUTE_NORMAL,
					(HANDLE)INVALID_HANDLE_VALUE))==INVALID_HANDLE_VALUE)
    {
        printf("Cann't found firewall device!\n");
        goto main_exit;
    }
	printf("Device %s opened, handle %p\n",fwDevice,hFw);

	// get current state
    if (!DeviceIoControl(	hFw,
							FW_IOCTL_GET_ONOFF,
							NULL,
							0,
							&tmp,
							sizeof(tmp),
							&BytesReturned,
							NULL))
    {
        printf("[FW_IOCTL_GET_ONOFF] failed!\n");
        goto main_exit;
    }
	printf("Firewall is %s\n",(tmp)?"ON":"OFF");

	// get current rules
    if (!DeviceIoControl(	hFw,
							FW_IOCTL_FILTER_GET,
							NULL,
							0,
							&getfwr,
							sizeof(getfwr),
							&BytesReturned,
							NULL))
    {
        printf("[FW_IOCTL_FILTER_GET] failed!\n");
        goto main_exit;
    }
	if (getfwr.num>=MAX_FWRULES)
	{
		printf("[FW_IOCTL_FILTER_GET] invalid entries return!\n");
        goto main_exit;
	}
	if (getfwr.num)
	{
		printf("Firewall rules table (%u records):\n",getfwr.num);
		prnRules(&getfwr);
	}
	else
	{
		printf("Firewall rules table empty\n");
	}
	printf("\nPress any key to continue...\n");
	getch();

	printf("Disable firewall... ");
    if (!DeviceIoControl(	hFw,
							FW_IOCTL_SET_OFF,
							NULL,
							0,
							NULL,
							0,
							&BytesReturned,
							NULL))
    {
        printf("[FW_IOCTL_SET_OFF] failed!\n");
        goto main_exit;
    }
	printf("ok\n");

	// *** setup new table (3 records)
	// first record: drop all tcp from 81.25.196.35 (www.st.com)
    setfwr.fwr[0].sIP=inet_addr("192.168.12.250");
    setfwr.fwr[0].sM=inet_addr("255.255.255.255");
    setfwr.fwr[0].dIP=ANY_IPv4;
    setfwr.fwr[0].dM=inet_addr("0.0.0.0");
    setfwr.fwr[0].sP=ANY_PORT;
    setfwr.fwr[0].dP=ANY_PORT;
    setfwr.fwr[0].ad=FW_RULE_DENY;
	// second record: drop IRC tcp from any ip
    setfwr.fwr[1].sIP=ANY_IPv4;
    setfwr.fwr[1].sM=inet_addr("0.0.0.0");
    setfwr.fwr[1].dIP=ANY_IPv4;
    setfwr.fwr[1].dM=inet_addr("0.0.0.0");
    setfwr.fwr[1].sP=htons(6667);
    setfwr.fwr[1].dP=htons(0);
	setfwr.fwr[1].ad=FW_RULE_DENY;
	// third record: allow ftp only from 192.168.12.0 & 192.168.12.7
    setfwr.fwr[2].sIP=inet_addr("192.168.12.0");
    setfwr.fwr[2].sM=inet_addr("0.0.0.7");
    setfwr.fwr[2].dIP=ANY_IPv4;
    setfwr.fwr[2].dM=inet_addr("0.0.0.0");
    setfwr.fwr[2].sP=ANY_PORT;
    setfwr.fwr[2].dP=htons(21);
	setfwr.fwr[2].ad=FW_RULE_ALLOW;
	// set num
	setfwr.num=3;

	printf("Set new rules table... ");
    if (!DeviceIoControl(	hFw,
							FW_IOCTL_FILTER_SET,
							&setfwr,
							sizeof(ULONG)+sizeof(FWR)*setfwr.num,	// || sizeof(setfwr)
							NULL,
							0,
							&BytesReturned,
							NULL))
    {
        printf("[FW_IOCTL_FILTER_SET] failed!\n");
        goto main_exit;
    }
	printf("ok\n");

	// get current rules
    if (!DeviceIoControl(	hFw,
							FW_IOCTL_FILTER_GET,
							NULL,
							0,
							&getfwr,
							sizeof(getfwr),
							&BytesReturned,
							NULL))
    {
        printf("[FW_IOCTL_FILTER_GET] failed!\n");
        goto main_exit;
    }
	if (getfwr.num>=MAX_FWRULES)
	{
		printf("[FW_IOCTL_FILTER_GET] invalid entries return!\n");
        goto main_exit;
	}
	if (getfwr.num)
	{
		printf("Firewall rules table (%u records):\n",getfwr.num);
		prnRules(&getfwr);
	}
	else
	{
		printf("Firewall rules table empty\n");
	}
	printf("\nPress any key to continue...\n");
	getch();
	printf("Enable firewall... ");
    if (!DeviceIoControl(	hFw,
							FW_IOCTL_SET_ON,
							NULL,
							0,
							NULL,
							0,
							&BytesReturned,
							NULL))
    {
        printf("[FW_IOCTL_SET_ON] failed!\n");
        goto main_exit;
    }
    printf("ok\n");

	printf("You can check tcp firewall filter from another application and\n");
	printf("then press any key for exit from this test\n");
	getch();

	printf("Disable firewall... ");
    if (!DeviceIoControl(	hFw,
							FW_IOCTL_SET_OFF,
							NULL,
							0,
							NULL,
							0,
							&BytesReturned,
							NULL))
    {
        printf("[FW_IOCTL_SET_OFF] failed!\n");
        goto main_exit;
    }
	printf("ok\n");

	// get current rules
    if (!DeviceIoControl(	hFw,
							FW_IOCTL_FILTER_GET,
							NULL,
							0,
							&getfwr,
							sizeof(getfwr),
							&BytesReturned,
							NULL))
    {
        printf("[FW_IOCTL_FILTER_GET] failed!\n");
        goto main_exit;
    }
	if (getfwr.num>=MAX_FWRULES)
	{
		printf("[FW_IOCTL_FILTER_GET] invalid entries return!\n");
        goto main_exit;
	}
	if (getfwr.num)
	{
		printf("Firewall rules table (%u records):\n",getfwr.num);
		prnRules(&getfwr);
	}
	else
	{
		printf("Firewall rules table empty\n");
	}
	printf("\nPress any key to continue...\n");getch();

	printf("Clear rules...");
    if (!DeviceIoControl(	hFw,
							FW_IOCTL_FILTER_CLEAR,
							NULL,
							0,
							NULL,
							0,
							&BytesReturned,
							NULL))
    {
        printf("[FW_IOCTL_FILTER_CLEAR] failed!\n");
        goto main_exit;
    }
	printf("ok\n");

	// get current rules
    if (!DeviceIoControl(	hFw,
							FW_IOCTL_FILTER_GET,
							NULL,
							0,
							&getfwr,
							sizeof(getfwr),
							&BytesReturned,
							NULL))
    {
        printf("[FW_IOCTL_FILTER_GET] failed!\n");
        goto main_exit;
    }
	if (getfwr.num>=MAX_FWRULES)
	{
		printf("[FW_IOCTL_FILTER_GET] invalid entries return!\n");
        goto main_exit;
	}
	if (getfwr.num)
	{
		printf("Firewall rules table (%u records):\n",getfwr.num);
		prnRules(&getfwr);
	}
	else
	{
		printf("Firewall rules table empty\n");
	}
	printf("\nPress any key to continue...\n");getch();

main_exit:
    if (hFw!=INVALID_HANDLE_VALUE) CloseHandle(hFw);
}
